import usePermissionStore from '@/stores/permission'
import useUserStore from '@/stores/user'
import { storeToRefs } from 'pinia'
import { isLoginTimeout } from '@/utils'
import type { RouteRecordRaw, Router } from 'vue-router'

// 白名单: 用户没有登录也可以访问的页面( 比如 404 页面 )
const whiteList = ['/login']

export const useRouterGuards = (router: Router) => {
  // 另一种写法:
  /**
   * 路由前置守卫: 就是一个在路由跳转之前执行的拦截器
   * @param to 目标路由对象
   * @param from 起始路由对象
   * @param next 是否要去 ?
   */
  router.beforeEach(async (to, _, next) => {
    const userStore = useUserStore()
    const permissionStore = usePermissionStore()
    const { token, hasUserInfo } = storeToRefs(userStore)
    if (token.value) {
      if (isLoginTimeout()) {
        // 登录超时( token 失效 ): 执行退出登录操作
        userStore.logout()
        ElMessage.error('登录超时')
        return Promise.reject(new Error('token 失效'))
      }

      // 用户已登录( token 未过期 ), 不能访问登录页面, 重定向到首页
      if (to.path === '/login') next('/')
      else {
        // ? 不缓存 userInfo，这样就可以实现在每次刷新页面时，动态添加权限路由
        if (!hasUserInfo.value) {
          // 请求用户数据
          const permission = (await userStore.getUserInfo())?.permission
          if (permission) {
            // 处理用户权限，筛选出需要添加的路由
            const filterRoutes = permissionStore.filterRoutes(permission.menus)
            // 循环添加对应的动态权限路由
            filterRoutes.forEach((route: RouteRecordRaw) => {
              router.addRoute(route)
            })
            // 添加完动态路由之后，需要进行一次主动跳转（让添加的动态路由生效）
            return next(to.path) // 正常跳转
          }
        }
        next() // 正常跳转
      }
    } else {
      if (whiteList.includes(to.path)) {
        // 用户未登录, 只可以访问白名单中的页面
        next()
      } else {
        // 用户未登录, 访问白名单之外的页面时, 跳转登录页面
        next('/login')
      }
    }
  })
}

// 路由前置守卫: 就是一个在路由跳转之前执行的拦截器
// router.beforeEach((to) => {
//   // 用户未登录, 跳转登陆页面
//   if (to.path !== '/login' && !token) return '/login'
//   // 用户已登录( token 未过期 ), 访问登录页面时, 重定向到首页
//   if (to.path === '/login' && token) {
//     return '/'
//   }
//   // 正常跳转
//   return
// })
