import path from 'path'
import { fileURLToPath, URL } from 'node:url'
import { defineConfig } from 'vite'
import vue from '@vitejs/plugin-vue'
import AutoImport from 'unplugin-auto-import/vite'
import Components from 'unplugin-vue-components/vite'
import { ElementPlusResolver } from 'unplugin-vue-components/resolvers'
import { createSvgIconsPlugin } from 'vite-plugin-svg-icons'
// vite-plugin-style-import 已经两年没维护了
// 可参考的替换库：https://github.com/onebay/vite-plugin-imp
import {
  createStyleImportPlugin,
  ElementPlusResolve
} from 'vite-plugin-style-import'
import { compression } from 'vite-plugin-compression2'

// https://vitejs.dev/config/
export default defineConfig({
  plugins: [
    vue(),
    // 设置 API 的自动导入（js/ts）
    AutoImport({
      resolvers: [ElementPlusResolver()]
    }),
    // 设置组件的自动导入（template）
    Components({
      // relative paths to the directory to search for components.
      // dirs: ['src/components/src'],
      // Glob patterns to match file names to be detected as components.
      // When specified, the `dirs` and `extensions` options will be ignored.
      // 现在不会自动导入 src/components 下的自定义组件了
      globs: ['src/components/src/*.{vue}'],
      // resolvers for custom components
      resolvers: [ElementPlusResolver()]
    }),
    // 通过 symbol 注册 svg 图标, 然后通过 use 组件的 href="#symbolId" 来引用注册可复用的 svg 图标
    // https://github.com/vbenjs/vite-plugin-svg-icons/blob/main/README.zh_CN.md
    createSvgIconsPlugin({
      // 指定需要缓存的图标文件夹( svg 图标存放目录 )
      iconDirs: [path.resolve(__dirname, './src/assets/icons/svg')],
      // 指定 symbolId 格式, name 是导入的 svg 文件对应的名称(不包含文件后最)
      symbolId: 'icon-[name]'
    }),
    // element-plus 的自动按需导入可以在使用组件的时候自动按需导入其对应的样式, 除了部分组件（反馈组件, 如 ElMessage 等）, 下面的插件就是为了自动按需导入剩余组件的样式
    createStyleImportPlugin({
      resolves: [ElementPlusResolve()],
      libs: [
        {
          libraryName: 'element-plus',
          esModule: true,
          // name：组件样式文件的名称
          resolveStyle: (name: string) => {
            // 返回值为组件的样式文件的路径
            // 手动按需引入样式示例：
            // import 'element-plus/theme-chalk/el-message.css'
            // 返回值样式文件在项目中的路径（import）
            return `element-plus/theme-chalk/${name}.css`
          }
        }
      ]
    }),
    // tip：如果想要服务器支持发送 gzip 文件给客户端（浏览器），需要配置 nginx 等启用 gzip
    // 启用后，当项目目录下有 gzip 压缩文件（.gz）时，优先发送 .gz 文件给客户端
    // 如果浏览器不支持 gzip 编码文件，则发送原来的 js 文件
    // 参考链接：https://www.zhihu.com/question/25871199
    compression() // 配置 gzip 压缩
  ],
  resolve: {
    alias: {
      '@': fileURLToPath(new URL('./src', import.meta.url))
    }
  },
  server: {
    proxy: {
      // 带选项写法：http://localhost:5173/api/bar -> http://jsonplaceholder.typicode.com/bar
      // 当 url 以 /api 开头时触发此代理
      '^/api': {
        target: 'https://api.imooc-admin.lgdsunday.club/api',
        changeOrigin: true,
        rewrite: (path) => path.replace(/^\/api/, '')
      },
      // 当 url 以 /prod-api 开头时触发此代理
      '^/prod-api': {
        target: 'https://imooc-admin.lgdsunday.club/prod-api',
        changeOrigin: true,
        rewrite: (path) => path.replace(/^\/prod-api/, '')
      }
    }
  },
  build: {
    // 有大于 1M 的包才报警告信息，默认值为 500（单位kb）
    chunkSizeWarningLimit: 1024,
    rollupOptions: {
      output: {
        // 独立分包
        manualChunks: {
          elementPlus: ['element-plus'],
          dayJs: ['dayjs'],
          axios: ['axios'],
          fuseJs: ['fuse.js'],
          xlsx: ['xlsx'],
          'toast-ui-editor': ['@toast-ui/editor'],
          'wangeditor-editor': ['@wangeditor/editor']
        }
      }
    }
  }
})
